/*  ---------------------------------------------------------
	MPI performance test suite
	Test for MPI transfer rates/latency
	Alexander N. Andreyev (alexander@vvv.srcc.msu.su)
	Laboratory of Parallel Information Technologies, SRCC
	Copyright (c) 1997-1999

    this file: test_shell.c (generic test shell)
    to build a specific test, please attach
    a module that implements transfer_test_operation, 
    e.g.: cc transf1.c test_shell.c -lmpi

    Last revision:  March, 16 / 1999
----------------------------------------------------------- */
   
#include "transfer.h"

static const char *copyright_message = "MPI Performance Test Suite (c) Alexander N. Andreyev, 1998";
extern const char test_description[];

static void* mpi_buf;
static const char *default_output_file_name = "transfer.dat";
static char output_file_name[256];
static int msglen_min = MIN_MESSAGE_LENGTH;
static int msglen_max = MAX_MESSAGE_LENGTH;
static int msglen_step = MESSAGE_INCREMENT;
static int msglen_multiplier = 1;
static int buf_size = 0;
static int TIMES = DEFAULT_TIMES;

void main(int argc,char* argv[]) 
{
	int i,msglen;
	
	int rank,pool_size,i_am_the_master;
	double t1,t2,rate,latency;
	void *sendbuf = NULL,*recvbuf = NULL;
	MPI_Status status;
	FILE *out = NULL;

        strcpy(output_file_name,default_output_file_name);

	MPI_Init(&argc,&argv);
	MPI_Comm_size(MPI_COMM_WORLD,&pool_size);
	MPI_Comm_rank(MPI_COMM_WORLD,&rank);

	if(pool_size != 2) {
		printf("Expected exactly 2 MPI processes\n");
		exit(1);
	}
	i_am_the_master = (rank == 0);	

/* parse command line */
	if(i_am_the_master)	
	for(i = 1; i < argc; i ++)
		switch(argv[i][0]) {
			case 'm': msglen_min = atoi(argv[i]+1); break;
			case 'M': msglen_max = atoi(argv[i]+1); break;
			case 's': msglen_step = atoi(argv[i]+1); msglen_multiplier = 1; break;
			case 'K': msglen_multiplier = atoi(argv[i]+1); msglen_step = 0; break;
			case 'b': buf_size = atoi(argv[i]+1); break;
			case 'T': TIMES = atoi(argv[i]+1); break;			
            case 'o': strcpy(output_file_name,argv[i]+1);  break;
			default: fprintf(stderr,"WARNING: unrecognized option: %s\n",argv[i]);
			break;
		}

	/* exchange parameters with the slave */
	
	if(i_am_the_master) {
	printf("messages: %d to %d, step %d; %d times; buffer = %d\n",msglen_min,msglen_max,msglen_step,TIMES,buf_size);		
		MPI_Send(&msglen_min,1,MPI_INT,1,TAG1,MPI_COMM_WORLD);
		MPI_Send(&msglen_max,1,MPI_INT,1,TAG1,MPI_COMM_WORLD);
		MPI_Send(&msglen_step,1,MPI_INT,1,TAG1,MPI_COMM_WORLD);
		MPI_Send(&msglen_multiplier,1,MPI_INT,1,TAG1,MPI_COMM_WORLD);
		MPI_Send(&buf_size,1,MPI_INT,1,TAG1,MPI_COMM_WORLD);		
		MPI_Send(&TIMES,1,MPI_INT,1,TAG1,MPI_COMM_WORLD);		
		}
	else {
		MPI_Recv(&msglen_min,1,MPI_INT,0,TAG1,MPI_COMM_WORLD,&status);
		MPI_Recv(&msglen_max,1,MPI_INT,0,TAG1,MPI_COMM_WORLD,&status);
		MPI_Recv(&msglen_step,1,MPI_INT,0,TAG1,MPI_COMM_WORLD,&status);
		MPI_Recv(&msglen_multiplier,1,MPI_INT,0,TAG1,MPI_COMM_WORLD,&status);
		MPI_Recv(&buf_size,1,MPI_INT,0,TAG1,MPI_COMM_WORLD,&status);		
		MPI_Recv(&TIMES,1,MPI_INT,0,TAG1,MPI_COMM_WORLD,&status);	
		}

if(buf_size != 0) {
	mpi_buf = malloc(buf_size);
	if(mpi_buf == NULL)
		{ printf("Failed to allocate MPI buffer\n"); exit(1); }

	MPI_Buffer_attach(mpi_buf,buf_size);
	}
	

	if(i_am_the_master) {
		        printf("%s\n\n",copyright_message);
		        printf("Running test: %s\n",test_description);

                out = fopen(output_file_name,"a+");
                if(out == NULL) 
                	printf("WARNING: Can't open or create %s, writing to stdout\n",output_file_name);
                else {
	                printf("Writing output to %s\n",output_file_name);
			        fprintf(out,"\n\nTest: %s\n",test_description);
					fprintf(out,"\nSize\tTransfer (MB/sec)\n\n");
                }

				printf("Size\tTransfer (MB/sec)\n\n");

                }
                
	for(msglen = msglen_min; msglen <= msglen_max; 
			msglen = msglen * msglen_multiplier + msglen_step) {

		if(msglen) {
        	sendbuf = malloc(msglen);
        	recvbuf = malloc(msglen);
                if(sendbuf == NULL || recvbuf == NULL) {
					fprintf(stderr,"Can't allocate %d bytes\n",msglen);
					exit(1);
                }
                }	
		
		MPI_Barrier(MPI_COMM_WORLD);
		
        t1 = Wtime();

		transfer_test_operation(sendbuf,recvbuf,msglen,rank,TIMES);
               	
		MPI_Barrier(MPI_COMM_WORLD);
		
		t2 = Wtime();

		if(i_am_the_master) {
			
		if(msglen != 0) {
			rate = ((double)msglen * 2.0 * (double) TIMES)/((t2 - t1) * (double) MEGABYTE);
	 		printf("%d\t%2.4g\n",msglen,rate);
            if(out != NULL)
            	fprintf(out,"%d\t%2.4g\n",msglen,rate);
		}
		else {
			latency = (t2 - t1) * 1000000 / ((double) TIMES * 2);
			printf("Latency: %g microseconds\n",latency);
			if(out != NULL)
				fprintf(out,"Latency: %g microseconds\n",latency);
		} /* end of if/else */
		
		fflush(stdout);
		fflush(out);
		} /* end of master's section */
	if(msglen) {
	free(sendbuf);
	free(recvbuf);
	}

	}  /* end of loop over message sizes */

        if(out != NULL) 
	        fclose(out);

	MPI_Finalize();
}
